#define exception behavior 
trap 
{ 
  trap { continue } 
  write-host -ForegroundColor Red "Unexpected Exception!`n`r" 
  write-host -ForegroundColor White ($_.invocationinfo.positionmessage -replace "`n") 
  0..100 | foreach { write-host -ForegroundColor White   ((gv -ErrorAction SilentlyContinue -scope $_ myinvocation).value.positionmessage -replace "`n") } 
  write-host -ForegroundColor Red "$($_.Exception)" 
  exit 1 
} 
# >>>> MCB Systems edit 06/18/2011:  Omit domain name from hostname if not domain-joined.
# >>>> See http://www.mcbsys.com/techblog/2011/06/hyper-v-certificate-will-expire-within-30-days
if ((gwmi win32_computersystem).partofdomain -eq $true) {
	$hostname = "$((gwmi win32_computersystem).dnshostname).$((gwmi win32_computersystem).domain)" 
} else {
	$hostname = "$((gwmi win32_computersystem).dnshostname)" 
}
# <<<< End MCB Systems edit
write-host "Host name found:" $hostname 
function CreateCert() 
{ 
    write-host "Creating a new certificate using makecert.exe" 
    .\makecert.exe -r -pe -n "CN=$hostname" -b 01/01/2005 -e 01/01/2050 -sr LocalMachine -ss My -a sha1 -sky exchange -eku 1.3.6.1.5.5.7.3.1 
} 
function FindCert() 
{ 
    $t = new-object System.DateTime(2049,1,1,1,10,10) 
    $certs = @(dir cert:\LocalMachine\My -recurse | ?{$_.subject -eq "CN=$hostname"} | ? { $_.NotAfter.CompareTo($t) -eq 1 }) 
    if($certs[0] -eq $null) 
    { 
        return $null; 
    } 
    if($certs.Length -ne 1) 
    { 
        write-warning "More than one certificate is found in store. Please don't run makecert.exe multiple times." 
    } 
    $certs[0]; 
} 
 
#Find the certificate of interest 
$cert = FindCert 
if($cert -eq $null) 
{ 
    CreateCert 
    $cert = FindCert; 
    if($cert -eq $null) 
    { 
        throw "Certificate Not Found error. Check if makecert.exe is successful or not" 
    } 
} 
write-host "Found certificate of interest:" 
write-host $cert | select NotBefore, NotAfter 
 
#tweak system settings to let VMMS use the certificate of interest. 
$thumbprint = $cert.Thumbprint   
$location = $cert.PrivateKey.CspKeyContainerInfo.UniqueKeyContainerName 
$folderlocation = gc env:ALLUSERSPROFILE 
$folderlocation = $folderlocation + "\Microsoft\Crypto\RSA\MachineKeys\"  
$filelocation = $folderlocation + $location  
icacls $filelocation /grant "*S-1-5-83-0:(R)"  
$thumbprint = $cert.Thumbprint  
reg add "HKLM\Software\Microsoft\Windows NT\CurrentVersion\Virtualization" /v "AuthCertificateHash" /f /t REG_BINARY /d $thumbprint 
 
#fix loopback case. 
$store = new-object System.Security.Cryptography.X509Certificates.X509Store("Root","LocalMachine")    
$store.open("MaxAllowed")     
$store.add($cert)     
$store.close()     
 
#restart vmms 
net stop vmms 
net start vmms 
 
# Wait for job completion 
function WaitForResult($ret) 
{ 
  if($ret.ReturnValue -eq 0) { return; } 
  if($ret.ReturnValue -ne 4096) 
  { 
    Throw "Error was returned from WMI call: $($ret.ReturnValue)"; 
  } 
  $timeout = 300; # 5 minute timeout 
  while($true) 
  { 
    $job = [wmi]$ret.job 
    if($job.JobState -eq 7) { return; } 
    if($job.JobState -gt 7) { throw "Error while processing WMI job! $($job | fl * | out-string)" } 
 
    if($timeout -le 0) { throw "Timeout while processing WMI job! $($job | fl * | out-string)" } 
 
    $timeout -= 5; 
    Sleep 5 
  } 
} 
 
# get all VMs in Running state. 
$vms = gwmi -n root\virtualization msvm_computersystem 
$vms = $vms | where {$_.Name -ne $env:computername} 
$vms = $vms | where {$_.EnabledState -eq 2} 
 
#Save/Restore for all running VMs 
foreach($vm in $vms) 
{ 
    if($vm -ne $null) 
    { 
        Write-Host "Doing Save/Restore for VM:" $vm.ElementName 
        WaitForResult  $vm.RequestStateChange(32769) 
        WaitForResult  $vm.RequestStateChange(2) 
    } 
} 